"""
Kailash Analysis Module.

Provides analysis tools for workflow optimization and conditional execution.
"""

from .conditional_branch_analyzer import ConditionalBranchAnalyzer

__all__ = ["ConditionalBranchAnalyzer"]
